@@ -1,11 +1,13 @@
-/* -*- indent-tabs-mode: nil; js-indent-level: 2; js-indent-level: 2 -*- */
-/* vim: set ts=8 sts=2 et sw=2 tw=80: */
-/*
-//@line 7 "/home/daniel/trees/bg-ng/toolkit/mozapps/downloads/nsHelperAppDlg.js"
-*/
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 const {utils: Cu, interfaces: Ci, classes: Cc, results: Cr} = Components;
 Cu.import("resource://gre/modules/Services.jsm");
+Cu.import("resource://gre/modules/AppConstants.jsm");
+Cu.import("resource://gre/modules/XPCOMUtils.jsm");
+XPCOMUtils.defineLazyModuleGetter(this, "EnableDelayHelper",
+                                  "resource://gre/modules/SharedPromptUtils.jsm");
 
 ///////////////////////////////////////////////////////////////////////////////
 //// Helper Functions
@@ -95,7 +97,7 @@ nsUnknownContentTypeDialogProgressListener.prototype = {
 const PREF_BD_USEDOWNLOADDIR = "browser.download.useDownloadDir";
 const nsITimer = Components.interfaces.nsITimer;
 
-let downloadModule = {};
+var downloadModule = {};
 Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
 Components.utils.import("resource://gre/modules/DownloadLastDir.jsm", downloadModule);
 Components.utils.import("resource://gre/modules/DownloadPaths.jsm");
@@ -226,7 +228,21 @@ nsUnknownContentTypeDialog.prototype = {
       // because the original one is definitely gone (and nsIFilePicker doesn't like
       // a null parent):
       gDownloadLastDir = this._mDownloadDir;
-      parent = Services.wm.getMostRecentWindow("");
+      let windowsEnum = Services.wm.getEnumerator("");
+      while (windowsEnum.hasMoreElements()) {
+        let someWin = windowsEnum.getNext();
+        // We need to make sure we don't end up with this dialog, because otherwise
+        // that's going to go away when the user clicks "Save", and that breaks the
+        // windows file picker that's supposed to show up if we let the user choose
+        // where to save files...
+        if (someWin != this.mDialog) {
+          parent = someWin;
+        }
+      }
+      if (!parent) {
+        Cu.reportError("No candidate parent windows were found for the save filepicker." +
+                       "This should never happen.");
+      }
     }
 
     Task.spawn(function() {
@@ -391,7 +407,22 @@ nsUnknownContentTypeDialog.prototype = {
     // is now caught properly in the caller of validateLeafName.
     var createdFile = DownloadPaths.createNiceUniqueFile(aLocalFolder);
 
-//@line 412 "/home/daniel/trees/bg-ng/toolkit/mozapps/downloads/nsHelperAppDlg.js"
+    if (AppConstants.platform == "win") {
+      let ext;
+      try {
+        // We can fail here if there's no primary extension set
+        ext = "." + this.mLauncher.MIMEInfo.primaryExtension;
+      } catch (e) { }
+
+      // Append a file extension if it's an executable that doesn't have one
+      // but make sure we actually have an extension to add
+      let leaf = createdFile.leafName;
+      if (ext && leaf.slice(-ext.length) != ext && createdFile.isExecutable()) {
+        createdFile.remove(false);
+        aLocalFolder.leafName = leaf + ext;
+        createdFile = DownloadPaths.createNiceUniqueFile(aLocalFolder);
+      }
+    }
 
     return createdFile;
   },
@@ -468,7 +499,25 @@ nsUnknownContentTypeDialog.prototype = {
       // want users to be able to autodownload .exe files.
       var rememberChoice = this.dialogElement("rememberChoice");
 
-//@line 509 "/home/daniel/trees/bg-ng/toolkit/mozapps/downloads/nsHelperAppDlg.js"
+      // Just because we have a content-type of application/octet-stream
+      // here doesn't actually mean that the content is of that type. Many
+      // servers default to sending text/plain for file types they don't know
+      // about. To account for this, the uriloader does some checking to see
+      // if a file sent as text/plain contains binary characters, and if so (*)
+      // it morphs the content-type into application/octet-stream so that
+      // the file can be properly handled. Since this is not generic binary
+      // data, rather, a data format that the system probably knows about,
+      // we don't want to use the content-type provided by this dialog's
+      // opener, as that's the generic application/octet-stream that the
+      // uriloader has passed, rather we want to ask the MIME Service.
+      // This is so we don't needlessly disable the "autohandle" checkbox.
+
+      // commented out to close the opening brace in the if statement.
+      // var mimeService = Components.classes["@mozilla.org/mime;1"].getService(Components.interfaces.nsIMIMEService);
+      // var type = mimeService.getTypeFromURI(this.mLauncher.source);
+      // this.realMIMEInfo = mimeService.getFromTypeAndExtension(type, "");
+
+      // if (type == "application/octet-stream") {
       if (shouldntRememberChoice) {
         rememberChoice.checked = false;
         rememberChoice.disabled = true;
@@ -488,25 +537,21 @@ nsUnknownContentTypeDialog.prototype = {
 
     this.mDialog.setTimeout("dialog.postShowCallback()", 0);
 
-    let acceptDelay = Services.prefs.getIntPref("security.dialog_enable_delay");
-    this.mDialog.document.documentElement.getButton("accept").disabled = true;
-    this._showTimer = Components.classes["@mozilla.org/timer;1"]
-                                .createInstance(nsITimer);
-    this._showTimer.initWithCallback(this, acceptDelay, nsITimer.TYPE_ONE_SHOT);
+    this.delayHelper = new EnableDelayHelper({
+      disableDialog: () => {
+        this.mDialog.document.documentElement.getButton("accept").disabled = true;
+      },
+      enableDialog: () => {
+        this.mDialog.document.documentElement.getButton("accept").disabled = false;
+      },
+      focusTarget: this.mDialog
+    });
   },
 
   notify: function (aTimer) {
     if (aTimer == this._showTimer) {
       if (!this.mDialog) {
         this.reallyShow();
-      } else {
-        // The user may have already canceled the dialog.
-        try {
-          if (!this._blurred) {
-            this.mDialog.document.documentElement.getButton("accept").disabled = false;
-          }
-        } catch (ex) {}
-        this._delayExpired = true;
       }
       // The timer won't release us, so we have to release it.
       this._showTimer = null;
@@ -590,30 +635,24 @@ nsUnknownContentTypeDialog.prototype = {
     }
   },
 
-  _blurred: false,
-  _delayExpired: false,
-  onBlur: function(aEvent) {
-    this._blurred = true;
-    this.mDialog.document.documentElement.getButton("accept").disabled = true;
-  },
-
-  onFocus: function(aEvent) {
-    this._blurred = false;
-    if (this._delayExpired) {
-      var script = "document.documentElement.getButton('accept').disabled = false";
-      this.mDialog.setTimeout(script, 250);
-    }
-  },
-
   // Returns true if opening the default application makes sense.
   openWithDefaultOK: function() {
     // The checking is different on Windows...
-//@line 659 "/home/daniel/trees/bg-ng/toolkit/mozapps/downloads/nsHelperAppDlg.js"
+    if (AppConstants.platform == "win") {
+      // Windows presents some special cases.
+      // We need to prevent use of "system default" when the file is
+      // executable (so the user doesn't launch nasty programs downloaded
+      // from the web), and, enable use of "system default" if it isn't
+      // executable (because we will prompt the user for the default app
+      // in that case).
+
+      //  Default is Ok if the file isn't executable (and vice-versa).
+      return !this.mLauncher.targetFileIsExecutable;
+    }
     // On other platforms, default is Ok if there is a default app.
     // Note that nsIMIMEInfo providers need to ensure that this holds true
     // on each platform.
     return this.mLauncher.MIMEInfo.hasDefaultHandler;
-//@line 664 "/home/daniel/trees/bg-ng/toolkit/mozapps/downloads/nsHelperAppDlg.js"
   },
 
   // Set "default" application description field.
@@ -634,9 +673,10 @@ nsUnknownContentTypeDialog.prototype = {
 
   // getPath:
   getPath: function (aFile) {
-//@line 687 "/home/daniel/trees/bg-ng/toolkit/mozapps/downloads/nsHelperAppDlg.js"
+    if (AppConstants.platform == "macosx") {
+      return aFile.leafName || aFile.path;
+    }
     return aFile.path;
-//@line 689 "/home/daniel/trees/bg-ng/toolkit/mozapps/downloads/nsHelperAppDlg.js"
   },
 
   // initAppAndSaveToDiskValues:
@@ -682,9 +722,9 @@ nsUnknownContentTypeDialog.prototype = {
       otherHandler.hidden = false;
     }
 
-    var useDefault = this.dialogElement("useSystemDefault");
     var openHandler = this.dialogElement("openHandler");
     openHandler.selectedIndex = 0;
+    var defaultOpenHandler = this.dialogElement("defaultHandler");
 
     if (this.mLauncher.MIMEInfo.preferredAction == this.nsIMIMEInfo.useSystemDefault) {
       // Open (using system default).
@@ -692,7 +732,8 @@ nsUnknownContentTypeDialog.prototype = {
     } else if (this.mLauncher.MIMEInfo.preferredAction == this.nsIMIMEInfo.useHelperApp) {
       // Open with given helper app.
       modeGroup.selectedItem = this.dialogElement("open");
-      openHandler.selectedIndex = 1;
+      openHandler.selectedItem = (otherHandler && !otherHandler.hidden) ?
+                                 otherHandler : defaultOpenHandler;
     } else {
       // Save to disk.
       modeGroup.selectedItem = this.dialogElement("save");
@@ -700,11 +741,10 @@ nsUnknownContentTypeDialog.prototype = {
 
     // If we don't have a "default app" then disable that choice.
     if (!openWithDefaultOK) {
-      var useDefault = this.dialogElement("defaultHandler");
-      var isSelected = useDefault.selected;
+      var isSelected = defaultOpenHandler.selected;
 
       // Disable that choice.
-      useDefault.hidden = true;
+      defaultOpenHandler.hidden = true;
       // If that's the default, then switch to "save to disk."
       if (isSelected) {
         openHandler.selectedIndex = 1;
@@ -871,7 +911,20 @@ nsUnknownContentTypeDialog.prototype = {
         // for the file to be saved to to pass to |saveToDisk| - otherwise
         // we must ask the user to pick a save name.
 
-//@line 937 "/home/daniel/trees/bg-ng/toolkit/mozapps/downloads/nsHelperAppDlg.js"
+        /*
+        var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
+        var targetFile = null;
+        try {
+          targetFile = prefs.getComplexValue("browser.download.defaultFolder",
+                                             Components.interfaces.nsILocalFile);
+          var leafName = this.dialogElement("location").getAttribute("realname");
+          // Ensure that we don't overwrite any existing files here.
+          targetFile = this.validateLeafName(targetFile, leafName, null);
+        }
+        catch(e) { }
+
+        this.mLauncher.saveToDisk(targetFile, false);
+        */
 
         // see @notify
         // we cannot use opener's setTimeout, see bug 420405
@@ -925,31 +978,24 @@ nsUnknownContentTypeDialog.prototype = {
   // Retrieve the pretty description from the file
   getFileDisplayName: function getFileDisplayName(file)
   {
-//@line 1004 "/home/daniel/trees/bg-ng/toolkit/mozapps/downloads/nsHelperAppDlg.js"
+    if (AppConstants.platform == "win") {
+      if (file instanceof Components.interfaces.nsILocalFileWin) {
+        try {
+          return file.getVersionInfoField("FileDescription");
+        } catch (e) {}
+      }
+    } else if (AppConstants.platform == "macosx") {
+      if (file instanceof Components.interfaces.nsILocalFileMac) {
+        try {
+          return file.bundleDisplayName;
+        } catch (e) {}
+      }
+    }
     return file.leafName;
   },
 
-  // chooseApp:  Open file picker and prompt user for application.
-  chooseApp: function() {
-//@line 1052 "/home/daniel/trees/bg-ng/toolkit/mozapps/downloads/nsHelperAppDlg.js"
-    var nsIFilePicker = Components.interfaces.nsIFilePicker;
-    var fp = Components.classes["@mozilla.org/filepicker;1"]
-                       .createInstance(nsIFilePicker);
-    fp.init(this.mDialog,
-            this.dialogElement("strings").getString("chooseAppFilePickerTitle"),
-            nsIFilePicker.modeOpen);
-
-    fp.appendFilters(nsIFilePicker.filterApps);
-
-    if (fp.show() == nsIFilePicker.returnOK && fp.file) {
-      // Remember the file they chose to run.
-      var localHandlerApp =
-        Components.classes["@mozilla.org/uriloader/local-handler-app;1"].
-                   createInstance(Components.interfaces.nsILocalHandlerApp);
-      localHandlerApp.executable = fp.file;
-      this.chosenApp = localHandlerApp;
-//@line 1069 "/home/daniel/trees/bg-ng/toolkit/mozapps/downloads/nsHelperAppDlg.js"
-
+  finishChooseApp: function() {
+    if (this.chosenApp) {
       // Show the "handler" menulist since we have a (user-specified)
       // application now.
       this.dialogElement("modeDeck").setAttribute("selectedIndex", "0");
@@ -958,7 +1004,10 @@ nsUnknownContentTypeDialog.prototype = {
       var otherHandler = this.dialogElement("otherHandler");
       otherHandler.removeAttribute("hidden");
       otherHandler.setAttribute("path", this.getPath(this.chosenApp.executable));
-      otherHandler.label = this.getFileDisplayName(this.chosenApp.executable);
+      if (AppConstants.platform == "win")
+        otherHandler.label = this.getFileDisplayName(this.chosenApp.executable);
+      else
+        otherHandler.label = this.chosenApp.name;
       this.dialogElement("openHandler").selectedIndex = 1;
       this.dialogElement("openHandler").setAttribute("lastSelectedItemID", "otherHandler");
 
@@ -972,6 +1021,74 @@ nsUnknownContentTypeDialog.prototype = {
       openHandler.selectedItem = this.dialogElement(lastSelectedID);
     }
   },
+  // chooseApp:  Open file picker and prompt user for application.
+  chooseApp: function() {
+    if (AppConstants.platform == "win") {
+      // Protect against the lack of an extension
+      var fileExtension = "";
+      try {
+        fileExtension = this.mLauncher.MIMEInfo.primaryExtension;
+      } catch(ex) {
+      }
+
+      // Try to use the pretty description of the type, if one is available.
+      var typeString = this.mLauncher.MIMEInfo.description;
+
+      if (!typeString) {
+        // If there is none, use the extension to
+        // identify the file, e.g. "ZIP file"
+        if (fileExtension) {
+          typeString =
+            this.dialogElement("strings").
+            getFormattedString("fileType", [fileExtension.toUpperCase()]);
+        } else {
+          // If we can't even do that, just give up and show the MIME type.
+          typeString = this.mLauncher.MIMEInfo.MIMEType;
+        }
+      }
+
+      var params = {};
+      params.title =
+        this.dialogElement("strings").getString("chooseAppFilePickerTitle");
+      params.description = typeString;
+      params.filename    = this.mLauncher.suggestedFileName;
+      params.mimeInfo    = this.mLauncher.MIMEInfo;
+      params.handlerApp  = null;
+
+      this.mDialog.openDialog("chrome://global/content/appPicker.xul", null,
+                              "chrome,modal,centerscreen,titlebar,dialog=yes",
+                              params);
+
+      if (params.handlerApp &&
+          params.handlerApp.executable &&
+          params.handlerApp.executable.isFile()) {
+        // Remember the file they chose to run.
+        this.chosenApp = params.handlerApp;
+      }
+    }
+    else {
+//@line 1086 "/home/glazou/trees/clean-base/toolkit/mozapps/downloads/nsHelperAppDlg.js"
+      var nsIFilePicker = Components.interfaces.nsIFilePicker;
+      var fp = Components.classes["@mozilla.org/filepicker;1"]
+                         .createInstance(nsIFilePicker);
+      fp.init(this.mDialog,
+              this.dialogElement("strings").getString("chooseAppFilePickerTitle"),
+              nsIFilePicker.modeOpen);
+
+      fp.appendFilters(nsIFilePicker.filterApps);
+
+      if (fp.show() == nsIFilePicker.returnOK && fp.file) {
+        // Remember the file they chose to run.
+        var localHandlerApp =
+          Components.classes["@mozilla.org/uriloader/local-handler-app;1"].
+                     createInstance(Components.interfaces.nsILocalHandlerApp);
+        localHandlerApp.executable = fp.file;
+        this.chosenApp = localHandlerApp;
+      }
+//@line 1104 "/home/glazou/trees/clean-base/toolkit/mozapps/downloads/nsHelperAppDlg.js"
+    }
+    this.finishChooseApp();
+  },
 
   // Turn this on to get debugging messages.
   debug: false,
