@@ -1,4 +1,4 @@
-//@line 2 "/home/glazou/trees/bgng/bluegriffon/app/profile/bluegriffon-prefs.js"
+#filter substitution
 
 pref("toolkit.defaultChromeURI", "chrome://bluegriffon/content/xul/bluegriffon.xul");
 pref("browser.chromeURL", "chrome://bluegriffon/content/xul/bluegriffon.xul");
@@ -10,19 +10,21 @@ pref("bluegriffon.singletonWindowType", "bluegriffon");
 // see https://developer.mozilla.org/en/XUL/prefwindow
 pref("browser.preferences.instantApply", true);
 
-//@line 16 "/home/glazou/trees/bgng/bluegriffon/app/profile/bluegriffon-prefs.js"
+#ifdef XP_MACOSX
+pref("toolbar.customization.usesheet", true); // true for Mac
+#else
 pref("toolbar.customization.usesheet", false); // false otherwise
-//@line 18 "/home/glazou/trees/bgng/bluegriffon/app/profile/bluegriffon-prefs.js"
+#endif
 
 /* debugging prefs */
 pref("browser.dom.window.dump.enabled", false);
 pref("javascript.options.showInConsole", false);
 pref("javascript.options.strict", false);
+pref("bluegriffon.console.showInvalidVariables", false);
 pref("nglayout.debug.disable_xul_cache", true);
 pref("nglayout.debug.disable_xul_fastload", true);
 
 pref("general.useragent.extra.mybrowser", "bluegriffon/1.8");
-pref("intl.locale.matchOS", true);
 pref("intl.accept_charsets", "iso-8859-1,*,utf-8");
 pref("browser.display.use_document_fonts", 1);
 
@@ -83,10 +85,12 @@ pref("bluegriffon.defaults.extension.text-html", "html");
 pref("editor.use_css", true);
 pref("bluegriffon.css.policy", "manual");
 pref("bluegriffon.css.prefix", "BG_");
+pref("bluegriffon.css.support.blink", true);
 pref("bluegriffon.css.support.gecko", true);
+pref("bluegriffon.css.support.servo", true);
 pref("bluegriffon.css.support.webkit", true);
-pref("bluegriffon.css.support.presto", true);
-pref("bluegriffon.css.support.trident", true);
+pref("bluegriffon.css.support.vivliostyle", true);
+pref("bluegriffon.css.support.weasyprint", true);
 
 pref("bluegriffon.prettyprint", true);
 pref("bluegriffon.encode_entity", "html");
@@ -147,30 +151,9 @@ pref("bluegriffon.tabs.position", "center");
 pref("bluegriffon.osx.dock-integration", true);
 pref("extensions.getAddons.cache.enabled", false);
 
-// +------------------------------------+
-// | Disabled for now, too late for 1.4 |
-// +------------------------------------+
-//
-// pref("app.update.enabled", true);
-// pref("app.update.auto", true);
-// pref("app.update.mode", 2);
-// pref("app.update.silent", false);
-// pref("app.update.url", "http://bluegriffon.org/update.php/%PRODUCT%/%VERSION%/%BUILD_TARGET%/update.xml");
-// pref("app.update.url.manual", "http://bluegriffon.org/pages/Download");
-// pref("app.update.url.details", "http://bluegriffon.org/pages/Download");
-// pref("app.update.interval", 86400);
-// pref("app.update.nagTimer.download", 86400);
-// pref("app.update.nagTimer.restart", 1800);
-// pref("app.update.timer", 600000);
-// pref("app.update.showInstalledUI", false);
-// pref("app.update.incompatible.mode", 0);
-// pref("app.update.log", false);
-
 pref("bluegriffon.css.colors.names.enabled", true);
 pref("bluegriffon.css.colors.type", "hex");
 
-pref("tipoftheday.openAtStartup", true);
-
 // make links absolute when copied
 pref("clipboard.absoluteLinks", true);
 
@@ -229,6 +212,6 @@ pref("print.print_edge_bottom", 0);
 pref("layout.css.flexbox.enabled", true);
 
 pref("general.useragent.locale", "en-US");
-pref("intl.locale.matchOS", false);
+pref("intl.locale.matchOS", true);
 
 pref("app.support.baseURL", "http://bluegriffon.org/");
